@@ -586,7 +586,8 @@ class GraphSet(object):
 
         If no argument is given, this method returns the number of
         graphs in `self`.  Otherwise, this method returns a new
-        GraphSet with graphs that have `size` edges.
+        GraphSet with graphs that have `size` edges; this usage of
+        `len(size)` is obsoleted, and use `graph_size(size)` instead.
 
         This method never raises OverflowError unlike built-in len(gs).
 
@@ -600,7 +601,7 @@ class GraphSet(object):
           >>> graph2 = [(1, 2), (1, 4)]
           >>> graph3 = [(1, 2), (1, 4), (2, 3)]
           >>> gs = GraphSet([graph1, graph2, graph3])
-          >>> gs.same_len(2)
+          >>> gs.len(2)
           GraphSet([[(1, 2), (1, 4)]])
 
         Args:
@@ -610,12 +611,13 @@ class GraphSet(object):
           The number of graphs, or a new GraphSet object.
 
         See Also:
-          __len__(), smaller(), larger()
+          __len__(), smaller(), larger(), graph_size()
+
         """
         if size is None:
             return self._ss.len()
         else:
-            return GraphSet(self._ss.len(size))
+            return self.graph_size(size)
 
     def __iter__(self):
         """Iterates over graphs.
@@ -1061,7 +1063,7 @@ class GraphSet(object):
           A new GraphSet object.
 
         See Also:
-          larger(), len()
+          larger(), graph_size()
         """
         return GraphSet(self._ss.smaller(size))
 
@@ -1085,10 +1087,36 @@ class GraphSet(object):
           A new GraphSet object.
 
         See Also:
-          smaller(), len()
+          smaller(), graph_size()
         """
         return GraphSet(self._ss.larger(size))
 
+    def graph_size(self, size):
+        """Returns a new GraphSet with `size` edges.
+
+        This method returns a new GraphSet with graphs that have
+        `size` edges.
+
+        Examples:
+          >>> graph1 = [(1, 2)]
+          >>> graph2 = [(1, 2), (1, 4)]
+          >>> graph3 = [(1, 2), (1, 4), (2, 3)]
+          >>> gs = GraphSet([graph1, graph2, graph3])
+          >>> gs.graph_size(2)
+          GraphSet([[(1, 2), (1, 4)]])
+
+        Args:
+          size: The number of edges in a graph.
+
+        Returns:
+          A new GraphSet object.
+
+        See Also:
+          smaller(), larger()
+
+        """
+        return GraphSet(self._ss.set_size(size))
+
     def complement(self):
         """Returns a new GraphSet with complement graphs of `self`.
 
@@ -1549,10 +1577,9 @@ class GraphSet(object):
             else:
                 edges.append(e)
         return GraphSet.converters['to_graph'](edges)
-
     @staticmethod
     def graphs(vertex_groups=None, degree_constraints=None, num_edges=None,
-               no_loop=False, graphset=None):
+               no_loop=False, graphset=None, linear_constraints=None):
         """Returns a GraphSet with graphs under given constraints.
 
         Examples: a set of paths from vertex 1 to vertex 6
@@ -1589,8 +1616,20 @@ class GraphSet(object):
           graphset: Optional.  A GraphSet object.  Graphs to be stored
             are selected from this object.
 
+          linear_constraints: Optional.  A list of linear constraints.
+            A linear constraint consists of weighted edges and
+            lower/upper bounds.  An edge weight is a positive or
+            negative number, which defaults to 1.  Weights of the edges
+            that are not included in the constraint are zeros.  For
+            instance, `linear_constraints=[([(1, 2, 0.6), (2, 5),
+            (3, 6, 1.2)], (1.5, 2.0))]`, feasible graph weights are
+            between 1.5 and 2.0, e.g., `[(1, 2), (2, 3), (3, 6)]` or
+            `[(1, 2), (2, 5), (5, 6)]`.
+            See graphillion/test/graphset.py in detail.
+
         Returns:
           A new GraphSet object.
+
         """
         graph = []
         for e in setset.universe():
@@ -1636,10 +1675,25 @@ class GraphSet(object):
 
         ss = None if graphset is None else graphset._ss
 
+        lc = None
+        if linear_constraints is not None:
+            lc = []
+            for c in linear_constraints:
+                expr = []
+                for we in c[0]:
+                    u = pickle.dumps(we[0])
+                    v = pickle.dumps(we[1])
+                    w = float(we[2]) if len(we) >= 3 else 1.0
+                    expr.append((u, v, w))
+                min = float(c[1][0])
+                max = float(c[1][1])
+                lc.append((expr, (min, max)))
+
         ss = _graphillion._graphs(graph=graph, vertex_groups=vg,
                                   degree_constraints=dc, num_edges=ne,
                                   num_comps=nc, no_loop=no_loop,
-                                  search_space=ss)
+                                  search_space=ss,
+                                  linear_constraints=lc)
         return GraphSet(ss)
 
     @staticmethod
@@ -1798,6 +1852,20 @@ class GraphSet(object):
                                degree_constraints=dc,
                                no_loop=True, graphset=graphset)
 
+    @staticmethod
+    def show_messages(flag=True):
+        """Enables/disables status messages.
+
+        Args:
+          flag: Optional.  True or False.  If True, status messages are
+          enabled.  If False, they are disabled (initial setting).
+
+        Returns:
+          The setting before the method call.  True (enabled) or
+          False (disabled).
+        """
+        return _graphillion._show_messages(flag)
+
     @staticmethod
     def _traverse(edges, traversal, source):
         neighbors = {}
@@ -1811,14 +1879,13 @@ class GraphSet(object):
             else:
                 neighbors[v].add(u)
         assert source in neighbors
+        vertices = set(neighbors.keys())
 
         sorted_edges = []
         queue_or_stack = []
         visited_vertices = set()
         u = source
         while True:
-            if u in visited_vertices:
-                continue
             visited_vertices.add(u)
             for v in sorted(neighbors[u]):
                 if v in visited_vertices:
@@ -1827,11 +1894,15 @@ class GraphSet(object):
             new_vertices = neighbors[u] - visited_vertices - set(queue_or_stack)
             queue_or_stack.extend(new_vertices)
             if not queue_or_stack:
-                break
+                if visited_vertices == vertices:
+                    break
+                else:
+                    queue_or_stack.append(min(vertices - visited_vertices))
             if traversal == 'bfs':
                 u, queue_or_stack = queue_or_stack[0], queue_or_stack[1:]
             else:
                 queue_or_stack, u = queue_or_stack[:-1], queue_or_stack[-1]
+            assert u not in visited_vertices
         assert set(edges) == set(sorted_edges)
         return sorted_edges
 
