@@ -68,6 +68,9 @@ class TestGraphSet(unittest.TestCase):
 
         self.assertRaises(KeyError, GraphSet.set_universe, [(1,2), (2,1)])
 
+        GraphSet.set_universe([(1,2), (3,4)])  # disconnected graph
+        self.assertEqual(GraphSet.universe(), [(1,2), (3,4)])
+
     def test_constructors(self):
         gs = GraphSet()
         self.assertTrue(isinstance(gs, GraphSet))
@@ -247,6 +250,58 @@ class TestGraphSet(unittest.TestCase):
         self.assertRaises(KeyError, GraphSet.graphs, vertex_groups=[[7]])
         self.assertRaises(KeyError, GraphSet.graphs, degree_constraints={7: 1})
 
+    def test_linear_constraints(self):
+        GraphSet.set_universe([(1, 2), (1, 4), (2, 3), (2, 5), (3, 6), (4, 5),
+                               (5, 6)])
+
+        gs = GraphSet.graphs(linear_constraints=[([(2, 3)], (2, 1))])
+        self.assertEqual(gs, GraphSet())
+
+        gs = GraphSet.graphs(linear_constraints=[([], (1, 2))])
+        self.assertEqual(gs, GraphSet())
+
+        gs = GraphSet.graphs(linear_constraints=[([(5, 6, -1.5)], (-1.5, 0))])
+        self.assertEqual(gs, GraphSet.graphs())
+
+        gs = GraphSet.graphs(linear_constraints=[([(1, 2, 3.14)], (0, float("inf")))])
+        self.assertEqual(gs, GraphSet.graphs())
+
+        gs = GraphSet.graphs(linear_constraints=[(GraphSet.universe(), (0, float("inf")))])
+        self.assertEqual(gs, GraphSet.graphs())
+
+        gs = GraphSet.graphs(linear_constraints=[(GraphSet.universe(), (7, 7))])
+        self.assertEqual(gs, GraphSet([GraphSet.universe()]))
+
+        gs = GraphSet.graphs(linear_constraints=[(GraphSet.universe(), (1, 7))])
+        self.assertEqual(gs, GraphSet.graphs() - GraphSet([[]]))
+
+        gs = GraphSet.graphs(linear_constraints=[([(2, 3, -1), (2, 5, 10), (4, 1),
+                                                   (3, 6), (4, 5, -1), (5, 6)],
+                                                  (10, 100))])
+        self.assertEqual(len(gs), 52)
+
+        gs = GraphSet.graphs(linear_constraints=[([(2, 3), (2, 5, -10), (4, 1, -1),
+                                                   (3, 6, -1), (4, 5), (5, 6, -1)],
+                                                  (-100, -10))])
+        self.assertEqual(len(gs), 52)
+
+        gs = GraphSet.graphs(linear_constraints=[([(1, 2)], (1, 1)),
+                                                 ([(1, 4)], (1, 2)),
+                                                 ([(2, 3)], (1, 3)),
+                                                 ([(2, 5)], (1, 4)),
+                                                 ([(4, 5)], (1, 6)),
+                                                 ([(5, 6)], (1, 7))])
+        self.assertEqual(len(gs), 2)
+
+    def test_show_messages(self):
+        a = GraphSet.show_messages()
+        b = GraphSet.show_messages(True)
+        self.assertTrue(b)
+        c = GraphSet.show_messages(False)
+        self.assertTrue(c)
+        d = GraphSet.show_messages(a)
+        self.assertFalse(d)
+
     def test_comparison(self):
         gs = GraphSet([g12])
         self.assertEqual(gs, GraphSet([g12]))
@@ -285,6 +340,8 @@ class TestGraphSet(unittest.TestCase):
         self.assertEqual(gs.smaller(3), GraphSet([g0, g1, g12, g14, g4]))
         self.assertTrue(isinstance(gs.larger(3), GraphSet))
         self.assertEqual(gs.larger(3), GraphSet([g1234]))
+        self.assertTrue(isinstance(gs.graph_size(3), GraphSet))
+        self.assertEqual(gs.graph_size(3), GraphSet([g123, g134]))
         self.assertTrue(isinstance(gs.len(3), GraphSet))
         self.assertEqual(gs.len(3), GraphSet([g123, g134]))
 
