@@ -41,6 +41,10 @@ WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 using graphillion::setset;
 using graphillion::Range;
+using graphillion::vertex_t;
+using graphillion::edge_t;
+using graphillion::weighted_edge_t;
+using graphillion::linear_constraint_t;
 using std::map;
 using std::pair;
 using std::set;
@@ -482,7 +486,7 @@ static PyObject* setset_len2(PySetsetObject* self, PyObject* args) {
     return PyLong_FromString(buf.data(), NULL, 0);
   } else if (PyInt_Check(obj)) {
     int len = PyLong_AsLong(obj);
-    RETURN_NEW_SETSET(self, self->ss->size(len));
+    RETURN_NEW_SETSET(self, self->ss->set_size(len));
   } else {
     PyErr_SetString(PyExc_TypeError, "not int");
     return NULL;
@@ -685,6 +689,16 @@ static PyObject* setset_larger(PySetsetObject* self, PyObject* io) {
   RETURN_NEW_SETSET(self, self->ss->larger(set_size));
 }
 
+static PyObject* setset_set_size(PySetsetObject* self, PyObject* io) {
+  CHECK_OR_ERROR(io, PyInt_Check, "int", NULL);
+  int set_size = PyLong_AsLong(io);
+  if (set_size < 0) {
+    PyErr_SetString(PyExc_ValueError, "not unsigned int");
+    return NULL;
+  }
+  RETURN_NEW_SETSET(self, self->ss->set_size(set_size));
+}
+
 static PyObject* setset_join(PySetsetObject* self, PyObject* other) {
   CHECK_SETSET_OR_ERROR(other);
   RETURN_NEW_SETSET2(self, other, _other, self->ss->join(*_other->ss));
@@ -888,6 +902,7 @@ static PyMethodDef setset_methods[] = {
   {"hitting", reinterpret_cast<PyCFunction>(setset_hitting), METH_NOARGS, ""},
   {"smaller", reinterpret_cast<PyCFunction>(setset_smaller), METH_O, ""},
   {"larger", reinterpret_cast<PyCFunction>(setset_larger), METH_O, ""},
+  {"set_size", reinterpret_cast<PyCFunction>(setset_set_size), METH_O, ""},
   {"flip", reinterpret_cast<PyCFunction>(setset_flip), METH_VARARGS, ""},
   {"join", reinterpret_cast<PyCFunction>(setset_join), METH_O, ""},
   {"meet", reinterpret_cast<PyCFunction>(setset_meet), METH_O, ""},
@@ -1025,17 +1040,19 @@ static PyObject* graphset_graphs(PyObject*, PyObject* args, PyObject* kwds) {
   static char s5[] = "num_comps";
   static char s6[] = "no_loop";
   static char s7[] = "search_space";
-  static char* kwlist[8] = {s1, s2, s3, s4, s5, s6, s7, NULL};
+  static char s8[] = "linear_constraints";
+  static char* kwlist[9] = {s1, s2, s3, s4, s5, s6, s7, s8, NULL};
   PyObject* graph_obj = NULL;
   PyObject* vertex_groups_obj = NULL;
   PyObject* degree_constraints_obj = NULL;
   PyObject* num_edges_obj = NULL;
   int num_comps = -1, no_loop = 0;
   PyObject* search_space_obj = NULL;
-  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|OOOiiO", kwlist, &graph_obj,
+  PyObject* linear_constraints_obj = NULL;
+  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|OOOiiOO", kwlist, &graph_obj,
                                    &vertex_groups_obj, &degree_constraints_obj,
                                    &num_edges_obj, &num_comps, &no_loop,
-                                   &search_space_obj))
+                                   &search_space_obj, &linear_constraints_obj))
     return NULL;
 
   vector<pair<string, string> > graph;
@@ -1067,9 +1084,10 @@ static PyObject* graphset_graphs(PyObject*, PyObject* args, PyObject* kwds) {
     graph.push_back(make_pair(e[0], e[1]));
   }
 
+  vector<vector<string> > vertex_groups_entity;
   vector<vector<string> >* vertex_groups = NULL;
   if (vertex_groups_obj != NULL && vertex_groups_obj != Py_None) {
-    vertex_groups = new vector<vector<string> >();
+    vertex_groups = &vertex_groups_entity;
     PyObject* i = PyObject_GetIter(vertex_groups_obj);
     if (i == NULL) return NULL;
     PyObject* uo;
@@ -1090,9 +1108,10 @@ static PyObject* graphset_graphs(PyObject*, PyObject* args, PyObject* kwds) {
     }
   }
 
+  map<string, Range> degree_constraints_entity;
   map<string, Range>* degree_constraints = NULL;
   if (degree_constraints_obj != NULL && degree_constraints_obj != Py_None) {
-    degree_constraints = new map<string, Range>();
+    degree_constraints = &degree_constraints_entity;
     PyObject* vo;
     PyObject* lo;
     Py_ssize_t pos = 0;
@@ -1118,8 +1137,10 @@ static PyObject* graphset_graphs(PyObject*, PyObject* args, PyObject* kwds) {
     }
   }
 
+  Range num_edges_entity;
   Range* num_edges = NULL;
   if (num_edges_obj != NULL && num_edges_obj != Py_None) {
+    num_edges = &num_edges_entity;
     vector<int> r;
     PyObject* i = PyObject_GetIter(num_edges_obj);
     PyObject* io;
@@ -1131,15 +1152,56 @@ static PyObject* graphset_graphs(PyObject*, PyObject* args, PyObject* kwds) {
       }
       r.push_back(PyInt_AsLong(io));
     }
-    num_edges = new Range(r[0], r[1], r[2]);
+    *num_edges = Range(r[0], r[1], r[2]);
   }
 
   setset* search_space = NULL;
   if (search_space_obj != NULL && search_space_obj != Py_None)
     search_space = reinterpret_cast<PySetsetObject*>(search_space_obj)->ss;
 
+  vector<linear_constraint_t> linear_constraints_entity;
+  vector<linear_constraint_t>* linear_constraints = NULL;
+  if (linear_constraints_obj != NULL && linear_constraints_obj != Py_None) {
+    linear_constraints = &linear_constraints_entity;
+    PyObject* i = PyObject_GetIter(linear_constraints_obj);
+    if (i == NULL) return NULL;
+    PyObject* co;
+    while ((co = PyIter_Next(i))) {
+      linear_constraints->push_back(linear_constraint_t());
+      linear_constraint_t& c = linear_constraints->back();
+      vector<weighted_edge_t>& expr = c.first;
+      pair<double,double>& range = c.second;
+      PyObject* lo = PySequence_GetItem(co, 0);
+      if (lo == NULL) return NULL;
+      PyObject* j = PyObject_GetIter(lo);
+      if (j == NULL) return NULL;
+      PyObject* eo;
+      while ((eo = PyIter_Next(j))) {
+        PyObject* uo = PySequence_GetItem(eo, 0);
+        if (uo == NULL || !PyString_Check(uo)) return NULL;
+        string u = PyString_AsString(uo);
+        PyObject* vo = PySequence_GetItem(eo, 1);
+        if (vo == NULL || !PyString_Check(vo)) return NULL;
+        string v = PyString_AsString(vo);
+        PyObject* wo = PySequence_GetItem(eo, 2);
+        if (wo == NULL || !PyFloat_Check(wo)) return NULL;
+        double w = PyFloat_AsDouble(wo);
+        expr.push_back(make_pair(make_pair(u, v), w));
+      }
+      PyObject* ro = PySequence_GetItem(co, 1);
+      if (ro == NULL) return NULL;
+      PyObject* r0o = PySequence_GetItem(ro, 0);
+      if (r0o == NULL || !PyFloat_Check(r0o)) return NULL;
+      range.first = PyFloat_AsDouble(r0o);
+      PyObject* r1o = PySequence_GetItem(ro, 1);
+      if (r1o == NULL || !PyFloat_Check(r1o)) return NULL;
+      range.second = PyFloat_AsDouble(r1o);
+    }
+  }
+
   setset ss = SearchGraphs(graph, vertex_groups, degree_constraints, num_edges,
-                           num_comps, no_loop, search_space);
+                           num_comps, no_loop, search_space,
+                           linear_constraints);
 
   PySetsetObject* ret = reinterpret_cast<PySetsetObject*>
       (PySetset_Type.tp_alloc(&PySetset_Type, 0));
@@ -1147,12 +1209,19 @@ static PyObject* graphset_graphs(PyObject*, PyObject* args, PyObject* kwds) {
   return reinterpret_cast<PyObject*>(ret);
 }
 
+static PyObject* graphset_show_messages(PySetsetObject* self, PyObject* obj) {
+  int ret = graphillion::ShowMessages(PyObject_IsTrue(obj));
+  if (ret) Py_RETURN_TRUE;
+  else Py_RETURN_FALSE;
+}
+
 static PyMethodDef module_methods[] = {
   {"load", reinterpret_cast<PyCFunction>(setset_load), METH_O, ""},
   {"loads", reinterpret_cast<PyCFunction>(setset_loads), METH_O, ""},
   {"_elem_limit", reinterpret_cast<PyCFunction>(setset_elem_limit), METH_NOARGS, ""},
   {"_num_elems", setset_num_elems, METH_VARARGS, ""},
   {"_graphs", reinterpret_cast<PyCFunction>(graphset_graphs), METH_VARARGS | METH_KEYWORDS, ""},
+  {"_show_messages", reinterpret_cast<PyCFunction>(graphset_show_messages), METH_O, ""},
   {NULL}  /* Sentinel */
 };
 
